#ifndef SHADER_PROGRAM_H
#define SHADER_PROGRAM_H

#include <vector>

#include "shader.h"

class ShaderProgram : public GLHandleObj {
	typedef GLHandleObj super;
protected:
	//internal list of shaders that were created by this class and linked to this shader object
	//for keeping track of memory and what not
	std::vector<Shader *> internalShaderList;

	//here's a list of ALL shaders who are attached to this program
	//useful for processing for dumping logs and what not
	//notice I'm only keeping the handle -- that way if the object dies i wont get any dangling pointer errors
	std::vector<GLhandleARB> allShaderList;
public:

	void create();
	virtual void destroy();

	void attach(GLhandleARB extHandle);
	void attach(const Shader &shader);

	//for the lazy, who don't want to manually create all their shader objects ...
	bool attachShader(const char *filename, int shaderType, const char *prefixString = NULL);
	bool attachVertexShader(const char *filename, const char *prefixString = NULL);
	bool attachFragmentShader(const char *filename, const char *prefixString = NULL);

	bool link();

	void getInfoLog(int logsize, int *res, char *log);
	void printLog();

	void use();
	static void useNone();

	int getUniformLocation(const char *name);

	//here's me taking matters into my own hands
	//and building off the opengl start
	//todo - template me.
	void setUniform1i(const char *name, int value);
	void setUniform1f(const char *name, float value);

};

#endif //SHADER_PROGRAM_H