#ifndef SHADER_SHADER_H
#define SHADER_SHADER_H

#include "glhandle.h"

//there's a const dilemma at play
//if we only allow read methods 'const' access
//but require write access for setSource and compile
//the user can still create a Shader object,
//use getHandle, and do the OpenGL setSource & compile themselves
//so might as well only read-only what doesn't modify 'handle'
class Shader : public GLHandleObj {
	typedef GLHandleObj super;
public:

	void create(int shaderType);
	void setSource(const char *buffer) const;
	void compile() const;

	//combines create(), setSource() and compile(), and any file-loading ops into one
	//prefixString is inserted at the beginning of the file.
	bool createFromSource(const char *filename, int shaderType, const char *prefixString = NULL);
	//this used to be createFile() - but i figure it has so little to do with files i'd restate the title

	//static methods:
	static const char *nameForType(int shaderType);
};

//common subclasses:

template<int SHADERTYPE>
class TemplateShader : public Shader {
	typedef Shader super;
public:
	void create() { super::create(SHADERTYPE); }
	bool createFromSource(const char *filename, const char *prefixString = NULL) {
		return super::createFromSource(filename, SHADERTYPE, prefixString);
	}
};

typedef TemplateShader<GL_VERTEX_SHADER_ARB> VertexShader;
typedef TemplateShader<GL_FRAGMENT_SHADER_ARB> FragmentShader;


#endif //SHADER_SHADER_H